# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import sys
from unittest import mock

from tackerclient.common import exceptions
from tackerclient.common import utils
from tackerclient import shell
from tackerclient.tacker import v1_0 as tackerV1_0
from tackerclient.tacker.v1_0 import TackerCommand
from tackerclient.tacker.v1_0.vnfm import vnf
from tackerclient.tests.unit import test_cli10
from tackerclient.tests.unit import test_utils

API_VERSION = "1.0"
FORMAT = 'json'
TOKEN = 'testtoken'
ENDURL = 'localurl'


class CLITestV10VmVNFJSON(test_cli10.CLITestV10Base):
    _RESOURCE = 'vnf'
    _RESOURCES = 'vnfs'
    _VNF_RESOURCES = 'vnf_resources'

    def setUp(self):
        plurals = {'vnfs': 'vnf',
                   'resources': 'resource'}
        super(CLITestV10VmVNFJSON, self).setUp(plurals=plurals)

    @mock.patch.object(TackerCommand, 'get_client')
    def _test_create_resource(self, resource, cmd, name, myid, args,
                              position_names, position_values, mock_get,
                              tenant_id=None, tags=None, admin_state_up=True,
                              extra_body=None, **kwargs):
        mock_get.return_value = self.client
        non_admin_status_resources = ['vnfd', 'vnf']
        if (resource in non_admin_status_resources):
            body = {resource: {}, }
        else:
            body = {resource: {'admin_state_up': admin_state_up, }, }
        if tenant_id:
            body[resource].update({'tenant_id': tenant_id})
        if tags:
            body[resource].update({'tags': tags})
        if extra_body:
            body[resource].update(extra_body)
        body[resource].update(kwargs)

        for i in range(len(position_names)):
            body[resource].update({position_names[i]: position_values[i]})
        ress = {resource:
                {self.id_field: myid}, }
        if name:
            ress[resource].update({'name': name})
        self.client.format = self.format
        resstr = self.client.serialize(ress)
        # url method body
        resource_plural = tackerV1_0._get_resource_plural(resource,
                                                          self.client)
        path = getattr(self.client, resource_plural + "_path")
        # Work around for LP #1217791. XML deserializer called from
        # MyComparator does not decodes XML string correctly.
        if self.format == 'json':
            _body = test_cli10.MyComparator(body, self.client)
        else:
            _body = self.client.serialize(body)
        with mock.patch.object(self.client.httpclient, 'request') as mock_req:
            mock_req.return_value = (test_cli10.MyResp(200), resstr)
            args.extend(['--request-format', self.format])
            args.extend(['--vnfd-id', 'vnfd'])
            cmd_parser = cmd.get_parser('create_' + resource)
            shell.run_command(cmd, cmd_parser, args)
            mock_req.assert_called_once_with(
                test_cli10.end_url(path, format=self.format), 'POST',
                body=_body,
                headers=test_utils.ContainsKeyValue('X-Auth-Token', TOKEN))
        mock_get.assert_any_call()

    def test_create_vnf_all_params(self):
        cmd = vnf.CreateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my_name'
        my_id = 'my-id'
        vnfd_id = 'vnfd'
        vim_id = 'vim_id'
        description = 'my-description'
        region_name = 'region'
        key = 'key'
        value = 'value'

        args = [
            name,
            '--vnfd-id', vnfd_id,
            '--vim-id', vim_id,
            '--description', description,
            '--vim-region-name', region_name,
            '--%s' % key, value]
        position_names = [
            'name',
            'vnfd_id',
            'vim_id',
            'description',
            'attributes',
        ]
        position_values = [
            name,
            vnfd_id,
            vim_id,
            description,
            {},
        ]
        extra_body = {key: value, 'placement_attr': {'region_name':
                                                     region_name}}
        self._test_create_resource(self._RESOURCE, cmd, name, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body)

    def test_create_vnf_with_vnfd_id(self):
        cmd = vnf.CreateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my_name'
        my_id = 'my-id'
        vnfd_id = 'vnfd'
        args = [
            name,
            '--vnfd-id', vnfd_id,
        ]
        position_names = ['name', 'vnfd_id', 'attributes']
        position_values = [name, vnfd_id, {}]
        self._test_create_resource(self._RESOURCE, cmd, name, my_id,
                                   args, position_names, position_values)

    def test_create_vnf_with_description_param(self):
        cmd = vnf.CreateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my_name'
        my_id = 'my-id'
        vnfd_id = 'vnfd'
        description = 'my-description'
        args = [
            name,
            '--vnfd-id', vnfd_id,
            '--description', description,
        ]
        position_names = ['name', 'vnfd_id', 'description',
                          'attributes']
        position_values = [name, vnfd_id, description, {}]
        self._test_create_resource(self._RESOURCE, cmd, None, my_id,
                                   args, position_names, position_values)

    def test_list_vnfs(self):
        cmd = vnf.ListVNF(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True)

    def test_list_vnfs_pagenation(self):
        cmd = vnf.ListVNF(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True)

    def test_show_vnf_id(self):
        cmd = vnf.ShowVNF(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id, args,
                                 ['id'])

    def test_show_vnf_id_name(self):
        cmd = vnf.ShowVNF(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_vnf(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        key = 'new_key'
        value = 'new-value'
        self._test_update_resource(self._RESOURCE, cmd, my_id,
                                   [my_id, '--%s' % key, value],
                                   {key: value})

    def test_vnf_update_param_file(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        key = 'key'
        value = 'new-value'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_param.yaml')
        args = [my_id, '--param-file', str(config_file)]
        extra_fields = {'attributes': {'param_values': {key: value}}}

        self._test_update_resource(self._RESOURCE, cmd, my_id, args,
                                   extra_fields)

    def test_vnf_update_config_file(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        description = 'Vim Description'
        vim_config = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_config.yaml')
        args = [
            name,
            '--config-file', vim_config,
            '--description', description]
        p_auth_url = 'http://1.2.3.4:5000'
        p_type = 'openstack'
        p_project_name = 'abc'
        p_username = 'xyz'
        p_project_domain_name = 'prj_domain_name'
        p_user_domain_name = 'user_domain_name'
        p_password = '12345'
        config = {'auth_url': p_auth_url, 'project_name': p_project_name,
                  'username': p_username,
                  'project_domain_name': p_project_domain_name,
                  'type': p_type, 'user_domain_name': p_user_domain_name,
                  'password': p_password}
        extra_body = {'description': description,
                      'attributes': {'config': config}}

        self._test_update_resource(self._RESOURCE, cmd, name, args, extra_body)

    def test_vnf_update_config(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        description = 'Vim Description'
        p_auth_url = 'https://1.2.3.4:6443'
        p_type = 'kubernetes'
        p_password = '12345'
        p_project_name = 'default'
        p_username = 'xyz'
        p_ssl_ca_cert = 'abcxyz'
        config = {'password': p_password, 'project_name': p_project_name,
                  'username': p_username, 'type': p_type,
                  'ssl_ca_cert': p_ssl_ca_cert, 'auth_url': p_auth_url}
        args = [
            name,
            '--description', description,
            '--config', str(config)]
        extra_body = {'description': description,
                      'attributes': {'config': config}}

        self._test_update_resource(self._RESOURCE, cmd, name, args, extra_body)

    def test_vnf_update_invalid_format_param_file(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'my-name'
        key = 'key'
        value = 'new-value'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_invalid_format_param.yaml')
        args = [my_id, '--param-file', str(config_file)]
        extra_fields = {'attributes': {'param_values': {key: value}}}

        self.assertRaises(exceptions.InvalidInput,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_fields)

    def test_vnf_update_empty_param_file(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'my-name'
        key = 'key'
        value = 'new-value'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_empty_param.yaml')
        args = [my_id, '--param-file', str(config_file)]
        extra_fields = {'attributes': {'param_values': {key: value}}}

        self.assertRaises(exceptions.InvalidInput,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_fields)

    def test_vnf_update_invalid_format_config_file(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        description = 'Vim Description'
        vim_config = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_invalid_format_config.yaml')
        args = [
            name,
            '--config-file', vim_config,
            '--description', description]
        p_auth_url = 'http://1.2.3.4:5000'
        p_type = 'openstack'
        p_project_name = 'abc'
        p_username = 'xyz'
        p_project_domain_name = 'prj_domain_name'
        p_user_domain_name = 'user_domain_name'
        p_password = '12345'
        config = {'auth_url': p_auth_url, 'project_name': p_project_name,
                  'username': p_username,
                  'project_domain_name': p_project_domain_name, 'type': p_type,
                  'user_domain_name': p_user_domain_name,
                  'password': p_password}
        extra_body = {'description': description,
                      'attributes': {'config': config}}

        self.assertRaises(exceptions.InvalidInput,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_body)

    def test_vnf_update_empty_config_file(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        description = 'Vim Description'
        vim_config = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_empty_config.yaml')
        args = [
            name,
            '--config-file', vim_config,
            '--description', description]
        extra_body = {'description': description}

        self.assertRaises(exceptions.InvalidInput,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_body)

    def test_vnf_update_invalid_format_config(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'vnfs/my-id'
        key = 'key'
        value = 'new-value'
        description = 'Vim Description'
        extra_fields = {'attributes': {'param_values': {key: value}}}
        config = 'test: : ]['
        args = [my_id,
                '--config', config,
                '--description', description]

        self.assertRaises(exceptions.InvalidInput,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_fields)

    def test_vnf_update_empty_config(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        description = 'Vim Description'
        config = {}
        args = [name,
                '--config', str(config),
                '--description', description]
        extra_body = {'description': description}

        self.assertRaises(exceptions.InvalidInput,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_body)

    def test_vnf_update_multi_args_config_configfile_paramfile(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        key = 'key'
        name = 'my-name'
        value = 'new-value'
        description = 'Vim Description'
        p_auth_url = 'https://1.2.3.4:6443'
        p_type = 'kubernetes'
        p_password = '12345'
        p_project_name = 'default'
        p_username = 'xyz'
        p_ssl_ca_cert = 'abcxyz'
        config = {'password': p_password, 'project_name': p_project_name,
                  'username': p_username, 'type': p_type,
                  'ssl_ca_cert': p_ssl_ca_cert, 'auth_url': p_auth_url}
        vim_config = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_config.yaml')
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_param.yaml')
        args = [my_id,
                '--config-file', str(vim_config),
                '--param-file', str(param_file),
                '--config', str(config),
                '--description', description]
        extra_fields = {'attributes': {'param_values': {key: value}}}

        self.assertRaises(BaseException,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_fields)

    def test_vnf_update_multi_args_configfile_paramfile(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my_name'
        my_id = 'my-id'
        description = 'Vim Description'
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_param.yaml')
        vim_config = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_config.yaml')
        args = [
            my_id,
            '--param-file', str(param_file),
            '--config-file', vim_config,
            '--description', description]
        extra_body = {'description': description}

        self.assertRaises(BaseException,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_body)

    def test_vnf_update_multi_args_config_configfile(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my_name'
        my_id = 'my-id'
        description = 'Vim Description'
        p_auth_url = 'https://1.2.3.4:6443'
        p_type = 'kubernetes'
        p_password = '12345'
        p_project_name = 'default'
        p_username = 'xyz'
        p_ssl_ca_cert = 'abcxyz'
        config = {'password': p_password, 'project_name': p_project_name,
                  'username': p_username, 'type': p_type,
                  'ssl_ca_cert': p_ssl_ca_cert, 'auth_url': p_auth_url}
        vim_config = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_config.yaml')
        args = [
            my_id,
            '--config-file', str(vim_config),
            '--config', str(config),
            '--description', description]
        extra_body = {'description': description}

        self.assertRaises(BaseException,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_body)

    def test_vnf_update_multi_args_config_paramfile(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my_name'
        my_id = 'my-id'
        description = 'Vim Description'
        p_auth_url = 'https://1.2.3.4:6443'
        p_type = 'kubernetes'
        p_password = '12345'
        p_project_name = 'default'
        p_username = 'xyz'
        p_ssl_ca_cert = 'abcxyz'
        config = {'password': p_password, 'project_name': p_project_name,
                  'username': p_username, 'type': p_type,
                  'ssl_ca_cert': p_ssl_ca_cert, 'auth_url': p_auth_url}
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_param.yaml')
        args = [
            my_id,
            '--param-file', str(param_file),
            '--config', str(config),
            '--description', description]
        extra_body = {'description': description}

        self.assertRaises(BaseException,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_body)

    def test_vnf_update_param_file_with_empty_dict(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'my-name'
        key = 'key'
        value = 'new-value'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/'
            'vnf_update_param_file_with_empty_dict.yaml')
        args = [my_id, '--param-file', str(config_file)]
        extra_fields = {'attributes': {'param_values': {key: value}}}

        self.assertRaises(exceptions.InvalidInput,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_fields)

    def test_vnf_update_config_file_with_empty_dict(self):
        cmd = vnf.UpdateVNF(test_cli10.MyApp(sys.stdout), None)
        name = 'my-name'
        description = 'Vim Description'
        vim_config = utils.get_file_path(
            'tests/unit/osc/samples/'
            'vnf_update_config_file_with_empty_dict.yaml')
        args = [
            name,
            '--config-file', vim_config,
            '--description', description]
        extra_body = {'description': description}

        self.assertRaises(exceptions.InvalidInput,
                          self._test_update_resource,
                          self._RESOURCE, cmd, name, args, extra_body)

    def test_delete_vnf(self):
        cmd = vnf.DeleteVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(self._RESOURCE, cmd, my_id, args)

    def test_delete_vnf_with_force(self):
        cmd = vnf.DeleteVNF(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id, '--force']
        self._test_delete_resource(self._RESOURCE, cmd, my_id, args)

    def test_list_vnf_resources(self):
        cmd = vnf.ListVNFResources(test_cli10.MyApp(sys.stdout), None)
        base_args = [self.test_id]
        response = [{'name': 'CP11', 'id': 'id1', 'type': 'NeutronPort'},
                    {'name': 'CP12', 'id': 'id2', 'type': 'NeutronPort'}]
        val = self._test_list_sub_resources(self._VNF_RESOURCES, 'resources',
                                            cmd, self.test_id,
                                            response_contents=response,
                                            detail=True, base_args=base_args)
        self.assertIn('id1', val)
        self.assertIn('NeutronPort', val)
        self.assertIn('CP11', val)

    def test_multi_delete_vnf(self):
        cmd = vnf.DeleteVNF(test_cli10.MyApp(sys.stdout), None)
        vnf_ids = 'vnf1 vnf2 vnf3'
        args = [vnf_ids]
        self._test_delete_resource(self._RESOURCE, cmd, vnf_ids, args)



# Copyright (c) 2012 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


EXT_NS = '_extension_ns'
XML_NS_V10 = 'http://openstack.org/tacker/api/v1.0'
XSI_NAMESPACE = "http://www.w3.org/2001/XMLSchema-instance"
XSI_ATTR = "xsi:nil"
XSI_NIL_ATTR = "xmlns:xsi"
TYPE_XMLNS = "xmlns:tacker"
TYPE_ATTR = "tacker:type"
VIRTUAL_ROOT_KEY = "_v_root"
ATOM_NAMESPACE = "http://www.w3.org/2005/Atom"
ATOM_XMLNS = "xmlns:atom"
ATOM_LINK_NOTATION = "{%s}link" % ATOM_NAMESPACE

TYPE_BOOL = "bool"
TYPE_INT = "int"
TYPE_FLOAT = "float"
TYPE_LIST = "list"
TYPE_DICT = "dict"


PLURALS = {'templates': 'template',
           'devices': 'device'}



# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ast
from unittest import mock

import fixtures
import yaml

from tackerclient.common import exceptions
from tackerclient.common import utils
from tackerclient.osc.v1.vnfm import vnf
from tackerclient.tests.unit.osc import base
from tackerclient.tests.unit.osc.v1.fixture_data import client


def _get_columns_vnf_parameter():

    columns = ['attributes', 'project_id']
    return columns


class TestVnfParameter(base.FixturedTestCase):
    client_fixture_class = client.ClientFixture

    def setUp(self):
        super(TestVnfParameter, self).setUp()
        self.url = client.TACKER_URL
        self.header = {'content-type': 'application/json'}
        self.app = mock.Mock()
        self.app_args = mock.Mock()
        self.client_manager = self.cs
        self.app.client_manager.tackerclient = self.client_manager


class TestUpdateVNF(TestVnfParameter):

    def setUp(self):
        super(TestUpdateVNF, self).setUp()
        self.useFixture(fixtures.MonkeyPatch(
            'tackerclient.tacker.v1_0.find_resourceid_by_name_or_id',
            self._find_resourceid))
        self.set_vnf = vnf.UpdateVNF(
            self.app, self.app_args, cmd_name='vnf set')

    def _find_resourceid(self, client, resource, name_or_id):
        return name_or_id

    def _cmd_parser(self, cmd_parser, sub_argv):
        _argv = sub_argv
        index = -1
        if '--' in sub_argv:
            index = sub_argv.index('--')
            _argv = sub_argv[:index]
        known_args, _values_specs = cmd_parser.parse_known_args(_argv)
        return known_args

    def _get_vnf_data(self, vnf_parameter):
        return tuple([vnf_parameter[key]
                     for key in sorted(vnf_parameter.keys())])

    def _take_action(self, args, extra_fields, get_client_called_count=1):
        cmd_par = self.set_vnf.get_parser("update_vnf")
        parsed_args = self._cmd_parser(cmd_par, args)
        body = {"vnf": extra_fields}
        body = str(body)
        project_id = {"tenant_id": "test-vnf-tenant_id"}
        extra_fields.update(project_id)
        json = {"vnf": extra_fields}

        self.requests_mock.register_uri(
            'PUT', self.url + '/v1.0/vnfs/my-id.json',
            json=json, headers=self.header)
        columns, data = (self.set_vnf.take_action(parsed_args))

        self.assertCountEqual(_get_columns_vnf_parameter(),
                              columns)
        self.assertEqual(get_client_called_count,
                         self.requests_mock.call_count)
        self.assertCountEqual(
            ast.literal_eval(self.requests_mock.last_request.body),
            ast.literal_eval(body))
        self.assertCountEqual(self._get_vnf_data(json['vnf']), data)

    def test_vnf_update_param_file(self):
        my_id = 'my-id'
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_param.yaml')
        args = [my_id, '--param-file', str(param_file)]
        extra_fields = {'attributes': {'param_values': {'key': 'new-value'}}}

        self._take_action(args, extra_fields)

    def test_vnf_update_config_file(self):
        my_id = 'my-id'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_config.yaml')
        p_auth_url = 'http://1.2.3.4:5000'
        p_project_name = 'abc'
        p_username = 'xyz'
        p_project_domain_name = 'prj_domain_name'
        p_type = 'openstack'
        p_user_domain_name = 'user_domain_name'
        p_password = '12345'
        args = [my_id, '--config-file', str(config_file)]
        config = {'auth_url': p_auth_url, 'project_name': p_project_name,
                  'username': p_username,
                  'project_domain_name': p_project_domain_name,
                  'type': p_type, 'user_domain_name': p_user_domain_name,
                  'password': p_password}
        extra_fields = {'attributes': {'config': config}}

        self._take_action(args, extra_fields)

    def test_vnf_update_config(self):
        my_id = 'my-id'
        p_auth_url = 'https://1.2.3.4:6443'
        p_type = 'kubernetes'
        p_password = '12345'
        p_project_name = 'default'
        p_username = 'xyz'
        p_ssl_ca_cert = 'abcxyz'
        config = {'password': p_password, 'project_name': p_project_name,
                  'username': p_username, 'type': p_type,
                  'ssl_ca_cert': p_ssl_ca_cert, 'auth_url': p_auth_url}
        config = yaml.dump(config)
        args = [my_id, '--config', str(config)]
        extra_fields = {'attributes': {'config': config}}

        self._take_action(args, extra_fields)

    def test_vnf_update_invalid_format_param_file(self):
        my_id = 'my-id'
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_invalid_format_param.yaml')
        args = [my_id, '--param-file', str(param_file)]
        extra_fields = {'attributes': {'param_values': None}}

        self.assertRaises(exceptions.InvalidInput,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_empty_param_file(self):
        my_id = 'my-id'
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_empty_param.yaml')
        args = [my_id, '--param-file', str(param_file)]
        extra_fields = {'attributes': {'param_values': None}}

        self.assertRaises(exceptions.InvalidInput,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_invalid_format_config_file(self):
        my_id = 'my-id'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_invalid_format_config.yaml')
        args = [my_id, '--config-file', str(config_file)]
        extra_fields = {'attributes': {'config': None}}

        self.assertRaises(exceptions.InvalidInput,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_empty_config_file(self):
        my_id = 'my-id'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_empty_config.yaml')
        args = [my_id, '--config-file', str(config_file)]
        extra_fields = {'attributes': {'config': None}}

        self.assertRaises(exceptions.InvalidInput,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_invalid_format_config(self):
        my_id = 'my_id'
        config = 'test: : ]['
        args = [my_id, '--config', config]
        extra_fields = {'attributes': {'config': None}}

        self.assertRaises(exceptions.InvalidInput,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_empty_config(self):
        my_id = 'my-id'
        config = ' '
        args = [my_id, '--config', config]
        extra_fields = {'attributes': {'config': None}}

        self.assertRaises(exceptions.InvalidInput,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_multi_args_config_configfile_paramfile(self):
        my_id = 'my-id'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_config.yaml')
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_param.yaml')
        p_auth_url = 'https://1.2.3.4:6443'
        p_type = 'kubernetes'
        p_password = '12345'
        p_project_name = 'default'
        p_username = 'xyz'
        p_ssl_ca_cert = 'abcxyz'
        config = {'password': p_password, 'project_name': p_project_name,
                  'username': p_username, 'type': p_type,
                  'ssl_ca_cert': p_ssl_ca_cert, 'auth_url': p_auth_url}
        config = yaml.dump(config)
        args = [my_id, '--config-file', str(config_file),
                '--config', str(config), '--param-file', str(param_file)]
        extra_fields = {'attributes': None}

        self.assertRaises(BaseException,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_multi_args_configfile_paramfile(self):
        my_id = 'my-id'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_config.yaml')
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_param.yaml')
        args = [my_id, '--param-file', str(param_file),
                '--config-file', str(config_file)]
        extra_fields = {'attributes': None}

        self.assertRaises(BaseException,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_multi_args_config_configfile(self):
        my_id = 'my-id'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_config.yaml')
        p_auth_url = 'https://1.2.3.4:6443'
        p_type = 'kubernetes'
        p_password = '12345'
        p_project_name = 'default'
        p_username = 'xyz'
        p_ssl_ca_cert = 'abcxyz'
        config = {'password': p_password, 'project_name': p_project_name,
                  'username': p_username, 'type': p_type,
                  'ssl_ca_cert': p_ssl_ca_cert, 'auth_url': p_auth_url}
        config = yaml.dump(config)
        args = [my_id, '--config-file', str(config_file),
                '--config', str(config)]
        extra_fields = {'attributes': None}

        self.assertRaises(BaseException,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_multi_args_config_paramfile(self):
        my_id = 'my-id'
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/vnf_update_param.yaml')
        p_auth_url = 'https://1.2.3.4:6443'
        p_type = 'kubernetes'
        p_password = '12345'
        p_project_name = 'default'
        p_username = 'xyz'
        p_ssl_ca_cert = 'abcxyz'
        config = {'password': p_password, 'project_name': p_project_name,
                  'username': p_username, 'type': p_type,
                  'ssl_ca_cert': p_ssl_ca_cert, 'auth_url': p_auth_url}
        config = yaml.dump(config)
        args = [my_id, '--param-file', str(param_file),
                '--config', str(config)]
        extra_fields = {'attributes': None}

        self.assertRaises(BaseException,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_param_file_with_empty_dict(self):
        my_id = 'my-id'
        param_file = utils.get_file_path(
            'tests/unit/osc/samples/'
            'vnf_update_param_file_with_empty_dict.yaml')
        args = [my_id, '--param-file', str(param_file)]
        extra_fields = {'attributes': None}

        self.assertRaises(exceptions.InvalidInput,
                          self._take_action,
                          args, extra_fields)

    def test_vnf_update_config_file_with_empty_dict(self):
        my_id = 'my-id'
        config_file = utils.get_file_path(
            'tests/unit/osc/samples/'
            'vnf_update_config_file_with_empty_dict.yaml')
        args = [my_id, '--config-file', str(config_file)]
        extra_fields = {'attributes': None}

        self.assertRaises(exceptions.InvalidInput,
                          self._take_action,
                          args, extra_fields)



# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from tackerclient.common.exceptions import InvalidInput
from tackerclient.tacker.v1_0.vnfm import vnfd
from tackerclient.tests.unit import test_cli10
from unittest.mock import mock_open
from unittest.mock import patch


class CLITestV10VmVNFDJSON(test_cli10.CLITestV10Base):
    _RESOURCE = 'vnfd'
    _RESOURCES = 'vnfds'

    def setUp(self):
        plurals = {'vnfds': 'vnfd'}
        super(CLITestV10VmVNFDJSON, self).setUp(plurals=plurals)

    @patch("tackerclient.tacker.v1_0.vnfm.vnfd.open",
           side_effect=mock_open(read_data="vnfd"),
           create=True)
    def test_create_vnfd_all_params(self, mo):
        cmd = vnfd.CreateVNFD(
            test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        name = 'my-name'
        attr_key = 'vnfd'
        attr_val = 'vnfd'
        args = [
            name,
            '--vnfd-file', 'vnfd-file'
        ]
        position_names = ['name']
        position_values = [name]
        extra_body = {
            'service_types': [{'service_type': 'vnfd'}],
            'attributes': {attr_key: attr_val},
        }
        self._test_create_resource(self._RESOURCE, cmd, None, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body)

    @patch("tackerclient.tacker.v1_0.vnfm.vnfd.open",
           side_effect=mock_open(read_data="vnfd"),
           create=True)
    def test_create_vnfd_with_mandatory_params(self, mo):
        cmd = vnfd.CreateVNFD(
            test_cli10.MyApp(sys.stdout), None)
        name = 'my_name'
        my_id = 'my-id'
        args = [name, '--vnfd-file', 'vnfd-file', ]
        position_names = ['name']
        position_values = [name]
        extra_body = {
            'service_types': [{'service_type': 'vnfd'}],
            'attributes': {'vnfd': 'vnfd'}
        }
        self._test_create_resource(self._RESOURCE, cmd, name, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body)

    @patch("tackerclient.tacker.v1_0.vnfm.vnfd.open",
           side_effect=mock_open(read_data=""),
           create=True)
    def test_create_vnfd_with_empty_file(self, mo):
        cmd = vnfd.CreateVNFD(
            test_cli10.MyApp(sys.stdout), None)
        name = 'my_name'
        my_id = 'my-id'
        args = [name, '--vnfd-file', 'vnfd-file', ]
        position_names = ['name']
        position_values = [name]
        extra_body = {
            'service_types': [{'service_type': 'vnfd'}],
            'attributes': {'vnfd': 'vnfd'}
        }
        err = None
        try:
            self._test_create_resource(self._RESOURCE, cmd, name, my_id,
                                       args, position_names, position_values,
                                       extra_body=extra_body)
        except InvalidInput:
            err = True
        self.assertEqual(True, err)

    def test_list_vnfds(self):
        cmd = vnfd.ListVNFD(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True,
                                  template_source='onboarded')

    def test_list_inline_vnfds(self):
        cmd = vnfd.ListVNFD(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True,
                                  template_source='inline')

    def test_list_all_vnfds(self):
        cmd = vnfd.ListVNFD(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True,
                                  template_source='all')

    def test_list_vnfds_pagenation(self):
        cmd = vnfd.ListVNFD(test_cli10.MyApp(sys.stdout), None)
        print(cmd)
        self._test_list_resources(self._RESOURCES, cmd, True,
                                  template_source='onboarded')

    def test_show_vnfd_id(self):
        cmd = vnfd.ShowVNFD(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id, args,
                                 ['id'])

    def test_show_vnfd_id_name(self):
        cmd = vnfd.ShowVNFD(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_delete_vnfd(self):
        cmd = vnfd.DeleteVNFD(
            test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(self._RESOURCE, cmd, my_id, args)

    def test_multi_delete_vnfd(self):
        cmd = vnfd.DeleteVNFD(
            test_cli10.MyApp(sys.stdout), None)
        vnfd_ids = 'my-id1 my-id2 my-id3'
        args = [vnfd_ids]
        self._test_delete_resource(self._RESOURCE, cmd, vnfd_ids, args)



# Copyright 2018 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import yaml

from osc_lib.command import command
from osc_lib import utils

from tackerclient.common import exceptions
from tackerclient.i18n import _
from tackerclient.osc import sdk_utils
from tackerclient.osc import utils as tacker_osc_utils
from tackerclient.tacker import v1_0 as tackerV10

_attr_map = (
    ('id', 'ID', tacker_osc_utils.LIST_BOTH),
    ('name', 'Name', tacker_osc_utils.LIST_BOTH),
    ('template_source', 'Template_Source',
     tacker_osc_utils.LIST_BOTH),
    ('description', 'Description', tacker_osc_utils.LIST_BOTH),
)

_VNFFGD = "vnffgd"

_formatters = {
    'template': tacker_osc_utils.format_dict_with_indention,
}


def _get_columns(item):
    column_map = {
        'tenant_id': 'project_id',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class CreateVNFFGD(command.ShowOne):
    _description = _("Create a new VNFFGD")

    def get_parser(self, prog_name):
        parser = super(CreateVNFFGD, self).get_parser(prog_name)
        parser.add_argument(
            'name', metavar='NAME',
            help=_('Name for VNFFGD'))
        parser.add_argument(
            '--tenant-id', metavar='TENANT_ID',
            help=_('The owner tenant ID or project ID'))
        parser.add_argument(
            '--vnffgd-file',
            required=True,
            help=_('YAML file with VNFFGD parameters'))
        parser.add_argument(
            '--description',
            help=_('Set a description for the VNFFGD'))
        return parser

    def args2body(self, parsed_args):
        body = {_VNFFGD: {}}
        vnffgd = None
        if not parsed_args.vnffgd_file:
            raise exceptions.InvalidInput(
                reason="Invalid input for vnffgd file")
        with open(parsed_args.vnffgd_file) as f:
            vnffgd = f.read()
            try:
                vnffgd = yaml.load(vnffgd, Loader=yaml.SafeLoader)
            except yaml.YAMLError as e:
                raise exceptions.InvalidInput(reason=e)
            if not vnffgd:
                raise exceptions.InvalidInput(reason="vnffgd file is empty")
            body[_VNFFGD]['template'] = {'vnffgd': vnffgd}
        tackerV10.update_dict(parsed_args, body[_VNFFGD],
                              ['tenant_id', 'name', 'description'])
        return body

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        vnffgd = client.create_vnffgd(self.args2body(parsed_args))
        display_columns, columns = _get_columns(vnffgd[_VNFFGD])
        data = utils.get_item_properties(
            sdk_utils.DictModel(vnffgd[_VNFFGD]),
            columns, formatters=_formatters)
        return (display_columns, data)


class DeleteVNFFGD(command.Command):
    _description = _("Delete VNFFGD(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteVNFFGD, self).get_parser(prog_name)
        parser.add_argument(
            _VNFFGD,
            metavar="<VNFFGD>",
            nargs="+",
            help=_("VNFFGD(s) to delete (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        failure = False
        deleted_ids = []
        failed_items = {}
        for resource_id in parsed_args.vnffgd:
            try:
                obj = tackerV10.find_resourceid_by_name_or_id(
                    client, _VNFFGD, resource_id)
                client.delete_vnffgd(obj)
                deleted_ids.append(resource_id)
            except Exception as e:
                failure = True
                failed_items[resource_id] = e
        if failure:
            msg = ''
            if deleted_ids:
                msg = (_('Successfully deleted %(resource)s(s):'
                         ' %(deleted_list)s') % {'deleted_list':
                                                 ', '.join(deleted_ids),
                                                 'resource': _VNFFGD})
            err_msg = _("\n\nUnable to delete the below"
                        " %s(s):") % _VNFFGD
            for failed_id, error in failed_items.items():
                err_msg += (_('\n Cannot delete %(failed_id)s: %(error)s')
                            % {'failed_id': failed_id,
                               'error': error})
            msg += err_msg
            raise exceptions.CommandError(message=msg)
        else:
            print((_('All specified %(resource)s(s) deleted successfully')
                   % {'resource': _VNFFGD}))
        return


class ListVNFFGD(command.Lister):
    _description = ("List (VNFFGD)s that belong to a given tenant.")

    def get_parser(self, prog_name):
        parser = super(ListVNFFGD, self).get_parser(prog_name)
        parser.add_argument(
            '--template-source',
            help=_("List VNFFGD with specified template source. Available \
                   options are 'onboarded' (default), 'inline' or 'all'"),
            action='store',
            default='onboarded')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        data = client.list_vnffgds()
        headers, columns = tacker_osc_utils.get_column_definitions(
            _attr_map, long_listing=None)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data[_VNFFGD + 's']))


class ShowVNFFGD(command.ShowOne):
    _description = _("Display VNFFGD details")

    def get_parser(self, prog_name):
        parser = super(ShowVNFFGD, self).get_parser(prog_name)
        parser.add_argument(
            _VNFFGD,
            metavar="<VNFFGD>",
            help=_("VNFFGD to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _VNFFGD, parsed_args.vnffgd)
        obj = client.show_vnffgd(obj_id)
        display_columns, columns = _get_columns(obj[_VNFFGD])
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_VNFFGD]),
            columns, formatters=_formatters)
        return (display_columns, data)


class ShowTemplateVNFFGD(command.ShowOne):
    _description = _("Display VNFFGD Template details")

    def get_parser(self, prog_name):
        parser = super(ShowTemplateVNFFGD, self).get_parser(prog_name)
        parser.add_argument(
            _VNFFGD,
            metavar="<VNFFGD>",
            help=_("VNFFGD to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.tackerclient
        obj_id = tackerV10.find_resourceid_by_name_or_id(
            client, _VNFFGD, parsed_args.vnffgd)
        obj = client.show_vnffgd(obj_id)
        data = utils.get_item_properties(
            sdk_utils.DictModel(obj[_VNFFGD]),
            ('template',),
            formatters=_formatters)
        data = (data or _('Unable to display VNFFGD template!'))
        return (('template',), data)



# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from tackerclient.tacker.v1_0.events import events
from tackerclient.tests.unit import test_cli10

API_VERSION = "1.0"
FORMAT = 'json'
TOKEN = 'testtoken'
ENDURL = 'localurl'


class CLITestV10EventJSON(test_cli10.CLITestV10Base):
    _EVT_RESOURCE = 'event'
    _EVT_RESOURCES = _EVT_RESOURCE + 's'
    _VNF_EVT_RESOURCE = "vnf_event"
    _VNF_EVT_RESOURCES = _VNF_EVT_RESOURCE + 's'
    _VNFD_EVT_RESOURCE = "vnfd_event"
    _VNFD_EVT_RESOURCES = _VNFD_EVT_RESOURCE + 's'
    _VIM_EVT_RESOURCE = "vim_event"
    _VIM_EVT_RESOURCES = _VIM_EVT_RESOURCE + 's'

    def setUp(self):
        plurals = {'events': 'event', 'vnf_events': 'vnf_event',
                   'vnfd_events': 'vnfd_event', 'vim_events': 'vim_event'}
        super(CLITestV10EventJSON, self).setUp(plurals=plurals)

    def test_list_events(self):
        cmd = events.ListResourceEvents(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._EVT_RESOURCES, cmd, True)

    def test_show_event_id(self):
        cmd = events.ShowEvent(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(self._EVT_RESOURCE, cmd, self.test_id, args,
                                 ['id'])

    def notest_list_vnf_events(self):
        # TODO(vishwanathj): Need to enhance _test_list_resources()
        # for supporting filters to get this test working
        cmd = events.ListVNFEvents(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._VNF_EVT_RESOURCES, cmd, True)

    def notest_list_vnfd_events(self):
        # TODO(vishwanathj): Need to enhance _test_list_resources()
        # for supporting filters to get this test working
        cmd = events.ListVNFDEvents(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._VNFD_EVT_RESOURCES, cmd, True)

    def notest_list_vim_events(self):
        # TODO(vishwanathj): Need to enhance _test_list_resources()
        # for supporting filters to get this test working
        cmd = events.ListVIMEvents(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._VIM_EVT_RESOURCES, cmd, True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import yaml

from oslo_serialization import jsonutils

from tackerclient.i18n import _
from tackerclient.tacker import v1_0 as tackerV10

_NSD = "nsd"


class ListNSD(tackerV10.ListCommand):
    """List NSDs that belong to a given tenant."""

    resource = _NSD
    list_columns = ['id', 'name', 'template_source', 'description']

    def get_parser(self, prog_name):
        parser = super(ListNSD, self).get_parser(prog_name)
        parser.add_argument(
            '--template-source',
            help=_("List NSD with specified template source. Available \
                   options are 'onboared' (default), 'inline' or 'all'"),
            action='store',
            default='onboarded')
        return parser

    def args2search_opts(self, parsed_args):
        search_opts = super(ListNSD, self).args2search_opts(parsed_args)
        template_source = parsed_args.template_source
        if parsed_args.template_source:
            search_opts.update({'template_source': template_source})
        return search_opts


class ShowNSD(tackerV10.ShowCommand):
    """Show information of a given NSD."""

    resource = _NSD


class CreateNSD(tackerV10.CreateCommand):
    """Create a NSD."""
    resource = _NSD
    remove_output_fields = ["attributes"]

    def add_known_arguments(self, parser):
        parser.add_argument('--nsd-file', help='Specify NSD file',
                            required=True)
        parser.add_argument(
            'name', metavar='NAME',
            help='Set a name for the NSD')
        parser.add_argument(
            '--description',
            help='Set a description for the NSD')

    def args2body(self, parsed_args):
        body = {self.resource: {}}
        nsd = None
        with open(parsed_args.nsd_file) as f:
            nsd = yaml.safe_load(f.read())
        tackerV10.update_dict(parsed_args, body[self.resource],
                              ['tenant_id', 'name', 'description'])
        if nsd:
            body[self.resource]['attributes'] = {'nsd': nsd}

        return body


class DeleteNSD(tackerV10.DeleteCommand):
    """Delete a given NSD."""
    resource = _NSD


class ShowTemplateNSD(tackerV10.ShowCommand):
    """Show template of a given NSD."""
    resource = _NSD

    def run(self, parsed_args):
        self.log.debug('run(%s)', parsed_args)
        template = None
        data = self.get_data(parsed_args)
        try:
            attributes_index = data[0].index('attributes')
            attributes_json = data[1][attributes_index]
            template = jsonutils.loads(attributes_json).get('nsd', None)
        except (IndexError, TypeError, ValueError) as e:
            self.log.debug('Data handling error: %s', str(e))
        print(template or _('Unable to display NSD template!'))



# Copyright 2014 Intel Corporation
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from tackerclient.common import utils
from tackerclient.tacker.v1_0.nfvo import vnffg
from tackerclient.tests.unit import test_cli10

API_VERSION = "1.0"
FORMAT = 'json'
TOKEN = 'testtoken'
ENDURL = 'localurl'


class CLITestV10VmVNFFGJSON(test_cli10.CLITestV10Base):
    _RESOURCE = 'vnffg'
    _RESOURCES = 'vnffgs'

    def setUp(self):
        plurals = {'vnffgs': 'vnffg'}
        super(CLITestV10VmVNFFGJSON, self).setUp(plurals=plurals)

    def test_create_vnffg_all_params(self):
        cmd = vnffg.CreateVNFFG(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        vnffgd_id = 'vnffgd'
        vnffg_name = 'fake-vnffg'
        vnf_mapping = 'VNFD1:VNF1'

        args = [
            vnffg_name,
            '--vnffgd-id', vnffgd_id,
            '--vnf-mapping', vnf_mapping,
            '--symmetrical']
        position_names = ['vnffgd_id', 'vnf_mapping', 'symmetrical']
        position_values = [vnffgd_id, {"VNFD1": "VNF1"}, True]
        extra_body = {'name': vnffg_name, 'attributes': {}}
        self._test_create_resource(self._RESOURCE, cmd, None, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body,
                                   get_client_called_count=2)

    def test_create_vnffg_with_mandatory_params(self):
        cmd = vnffg.CreateVNFFG(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        vnffg_name = 'fake-vnffg'
        vnffgd_id = 'vnffgd'
        args = [
            vnffg_name,
            '--vnffgd-id', vnffgd_id,
        ]
        position_names = ['vnffgd_id']
        position_values = [vnffgd_id]
        extra_body = {'symmetrical': False, 'name': vnffg_name,
                      'attributes': {}}
        self._test_create_resource(self._RESOURCE, cmd, vnffg_name, my_id,
                                   args, position_names, position_values,
                                   extra_body=extra_body,
                                   get_client_called_count=2)

    def test_list_vnffgs(self):
        cmd = vnffg.ListVNFFG(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True)

    def test_list_vnffgs_pagenation(self):
        cmd = vnffg.ListVNFFG(test_cli10.MyApp(sys.stdout), None)
        self._test_list_resources(self._RESOURCES, cmd, True)

    def test_show_vnffg_id(self):
        cmd = vnffg.ShowVNFFG(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id, args,
                                 ['id'])

    def test_show_vnffg_id_name(self):
        cmd = vnffg.ShowVNFFG(test_cli10.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(self._RESOURCE, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_vnffg(self):
        cmd = vnffg.UpdateVNFFG(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        update_vnffg = utils.get_file_path(
            'tests/unit/vm/samples/vnffg_update_file.yaml')
        vnf_mapping = 'VNFD1:VNF1'
        args = [
            my_id,
            '--vnf-mapping', vnf_mapping,
            '--vnffgd-template', str(update_vnffg),
            '--symmetrical'
        ]
        extra_fields = {
            "vnf_mapping": {"VNFD1": "VNF1"},
            "vnffgd_template": "abcxyz",
            "symmetrical": True
        }
        self._test_update_resource(self._RESOURCE, cmd, my_id,
                                   args, extra_fields,
                                   get_client_called_count=2)

    def test_delete_vnffg_without_force(self):
        cmd = vnffg.DeleteVNFFG(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(self._RESOURCE, cmd, my_id, args)

    def test_delete_vnffg_with_force(self):
        cmd = vnffg.DeleteVNFFG(test_cli10.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id, '--force']
        self._test_delete_resource(self._RESOURCE, cmd, my_id, args)
